import { SMouseEvent, SUndoStack } from "@saga-web/base";
import { SGraphScene } from '@saga-web/graph/lib';
import { SPoint } from '@saga-web/draw/lib';
import { SFloorParser, SImageItem, STextItem, SLineItem, SPolylineItem, SItemStatus, ItemOrder } from "@saga-web/big";
import { SGraphItem, SGraphPointListInsert, SGraphPointListDelete, SGraphPointListUpdate, SGraphAddCommand } from "@saga-web/graph/lib";
import { SPolygonItem } from "./SPolygonItem";
import { TipelineItem } from "@/lib/TipelineItem";


/**
 * 在线绘图
 *
 * @author 韩耀龙
 */

export class EditScence extends SGraphScene {

    undoStack = new SUndoStack();
    /** 命令 1 绘制直线 */
    private cmd = 0;
    /** 获取当前状态 */
    get getCmd(): number {
        return this.cmd;
    }
    /** 编辑当前状态 */
    set setCmd(cmd: number) {
        if (cmd == 0) {
            this.grabItem = null;
        }
        this.cmd = cmd;
        if (this.focusItem) {
            // 取消操作
            this.focusItem.cancelOperate();
            this.focusItem = null;
            // this.selectContainer.
        }
        if (this.view) {
            this.view.update();
        }
    };
    /** 当前选中焦点Item */
    focusItem: SGraphItem | null = null;

    constructor() {
        super();

    }

    /**
     * 增加线段item
     */
    addLine(event: SMouseEvent): boolean {
        const item = new SLineItem(null, new SPoint(event.x, event.y));
        this.addItem(item);
        item.connect("finishCreated", this, this.finishCreated);
        item.zOrder = ItemOrder.lineOrder
        this.grabItem = item;
        this.undoStack.push(new SGraphAddCommand(this, item));
        // item.connect("onMove", this, this.onItemMove.bind(this));
        return true
    }

    /**
     * 增加折线item
     */
    addPolylineItem(event: SMouseEvent): boolean {
        const point = new SPoint(event.x, event.y)
        const item = new TipelineItem(null, [point]);
        //设置状态
        item.status = SItemStatus.Create;
        item.zOrder = ItemOrder.polylineOrder
        this.addItem(item);
        item.connect("finishCreated", this, this.finishCreated);
        // this.undoStack.push(new SGraphAddCommand(this, item));
        this.grabItem = item;
        this.focusItem = item;
        console.log(this)
        return true
    }

    /**
     * 增加多边形item
     */
    addPolygonItem(event: SMouseEvent): void {
        //创建item
        const Polylines = new SPolygonItem(null);
        Polylines.selectable = true;
        //设置状态
        Polylines.setStatus = SItemStatus.Create;
        const point = new SPoint(event.x, event.y);
        Polylines.zOrder = ItemOrder.polygonOrder;
        Polylines.setPointList = [point];
        this.addItem(Polylines);
        Polylines.connect("finishCreated", this, this.finishCreated);
        this.grabItem = Polylines;
        this.focusItem = Polylines;
    }

    /**
     * 增加图片Item
     */
    addImgItem(event: SMouseEvent) {
        const url = require('./../../assets/logo.png')
        const item = new SImageItem(null);
        item.url = url;
        item.zOrder = ItemOrder.imageOrder;
        item.selectable = true;
        item.moveTo(event.x, event.y);
        this.addItem(item);
        this.grabItem == null;
        this.focusItem = item;
        this.cmd = 0;
    }

    /**
     * 增加文字item
     */
    addTextItem(event: SMouseEvent): void {
        const item = new STextItem(null, '请在右侧属性栏输入文字！');
        item.moveTo(event.x, event.y);
        item.selectable = true;
        item.zOrder = ItemOrder.textOrder;
        this.addItem(item);
        this.grabItem == null
        this.cmd = 0;
    }

    /**
     * 增加图标
     */
    addIconItem(): void {

    }

    /**
     * 更改item对应属性
     */
    editItemStatus(): void {

    }

    /**
    * 删除指定item
    */
    deleiteItemStatus(): void {

    }

    /**
     * 对齐指定item
     */
    alignItem(): void {

    }

    /**
     * 提取item
     */
    extractItem(): void {
    }

    /**
     * 锁住item
     */
    lockItem(): void {
    }

    /**
     * 执行取消操作
     */
    redo(): void {
        this.undoStack.undo();
    }

    /**
    * 执行重做操作执行
    */
    undo(): void {
        this.undoStack.redo();
    }
    /**
     * 完成事件创建的回调函数
     */
    finishCreated(item: any) {
        console.log(3333333333333)
        this.setCmd = 0;
        this.focusItem = item;
        this.selectContainer.toggleItem(item)
    }
    ////////////////////////
    //  以下为鼠标键盘操作事件
    onMouseDown(event: SMouseEvent): any {
        if (this.grabItem) {
            return this.grabItem.onMouseDown(event);
        }
        switch (this.cmd) {
            case 1:
                this.addLine(event);
                break;
            case 2:
                this.addTextItem(event);
                break;
            case 3:
                this.addImgItem(event)
                break;
            case 4:
                this.addPolygonItem(event);
                break;
            case 5:
                this.addPolylineItem(event);
                break;
            default:
                return super.onMouseDown(event);
        }
    }

    /**
     * 键盘事件
     *
     * @param	event         事件参数
     * @return	boolean
     */

    onKeyDown(event: KeyboardEvent): any {
        if (this.grabItem) {
            this.grabItem.onKeyDown(event);
        }
        // if (event.key == "Enter") {
        //     this.cmd = 0
        // }
        return false
    }
}